\documentclass{article}
\usepackage{cp0708t}
\usepackage{palatino}
\usepackage{a4}
\usepackage[draft]{fixme}

%include polycode.fmt 
%format fst = "\p1"
%format p1  = "\p1"
%format snd = "\p2"
%format p2  = "\p2"
%format Left = "i_1"
%format Right = "i_2"
%format i1 = "i_1"
%format i2 = "i_2"
%format >< = "\times"
%format |-> = "\mapsto"
%format . = "\comp"
%format -|- = "+"
%format Either a b = a "+" b
%format (either a b) = "\alt{" a "}{" b "}"
%format (either (a) (b)) = "\alt{" a "}{" b "}"
%format (uncurry f) = "\uncurry{" f "}"
%format (uncurry f) = "\uncurry{" f "}"
%format (const f) = "\underline{" f "}"
%format cata = "\hcata"
% -- desactivados:
%%format (uncurry (f)) = "\uncurry{" f "}"

\title{
		    Clculo de Programas
\\
		Trabalho Terico-Prtico
\\
		LCC --- Ano Lectivo de 2007/08
}

\author{
		J.N. Oliveira \& O.M. Pacheco
\\
		\dium
\\
		Universidade do Minho
}


\date\mydate

\makeindex

\begin{document}

\maketitle

\tableofcontents

\newpage

\section{Prembulo}

A disciplina de Clculo de Programas tem como objectivo principal ensinar a
programao de computadores como uma disciplina cientfica. Para isso parte-se de uma
lgebra da programao (conjunto de leis universais e seus corolrios) e aplica-se
essa lgebra a problemas de programao por resoluo de equaes cujas
incgnitas so os prprios programas que se querem obter.

Por razes de ordem pedaggica, restringe-se a aplicao desta cincia ao desenvolvimento
de programas funcionais e usa-se a linguagem \Haskell\ como veculo de programao~\footnote{%
A lgebra da programao estende-se  lgica em geral, encarada como liguagem para especificar problemas
e construir solues. Esta continuao desta disciplina aborda-se na unidade curricular
\emph{Mtodos Formais em Engenharia de Software} do Mestado em Informtica desta Universidade,
cf.
\begin{quote}
\texttt{http://www.di.uminho.pt/ensino/formacao-avancada/metodos-formais-em-engenharia-de-software}.
\end{quote}
}.

O presente trabalho tem por objectivo concretizar na prtica os
objectivos da disciplina acima enunciados: (a) por um lado, colocar os alunos perante
problemas de programao que devero ser resolvidos em \haskell; (b) por outro lado,
convidar os alunos a documentarem esses mesmos programas com os clculos que
precederam a sua codificao.
H ainda um terceiro objectivo: o de ensinar a produzir textos tcnico-cientficos de qualidade.

Para cumprir de forma integrada e simples os objectivos acima vamos recorrer a uma tcnica de
programa\-o dita \litp{\emph{literria}} \cite{Kn92}, cujo princpio base  o seguinte:
\emph{a documentao de um programa deve coincidir com ele prprio}. Por outras palavras,
o cdigo fonte e a sua documentao devero constar do mesmo documento (ficheiro).
O texto que est a ler  um exemplo de \litp{programao literria}, como adiante se referir.

\section{Como realizar o trabalho}
Este trabalho terico-prtico deve ser realizado por grupos com um mximo
de trs alunos. Os detalhes da avaliao (datas para submisso do relatrio e sua defesa oral)
so os que constam da pgina da disciplina na \emph{internet}, onde tambm se encontra o
ficheiro original do presente texto.
Recomenda-se uma abordagem equilibrada e participativa dos membros do
grupo de trabalho por forma a poderem responder s questes que sero colocadas
na defesa oral do relatrio.

Em que consiste, ento, o \emph{relatrio} a que se refere o pargrafo anterior?
Dever ser um texto de \litp{\emph{programao literria}} \cite{Kn92} que responda aos
requisitos e questes de programao que constam deste enunciado, mais adiante.

\section{Como documentar cdigo \Haskell}
Na programao cientfica, como  aquela que se ensina nesta disciplina,
 to importante produzir um bom programa, ou pacote de programas
(vulg.\ \emph{aplicao de `software'}) como produzir evidncia sobre a forma como foi
concebido (decises tomadas, etc).
Este aspecto faz com que a \emph{documentao} sobre um programa seja
tanto ou mais valiosa que o programa propriamente dito.
(Essa documentao , afinal, o mais precioso e duradouro bem de que uma empresa
dispe sempre que um autor de um produto de `software' seu a abandona.)

Um erro clssico dos hbitos tradicionais de programao  o de se escrever
a documentao s depois do respectivo programa \emph{estar pronto}.
Por via de regra, a documentao escrita nessa fase tardia  omissa e de m qualidade.
Isto porque muitos detalhes do desenvolvimento j esto esquecidos nessa altura, sendo
muitas vezes tudo escrito \emph{a correr}.

Todos so unnimes em recomendar que a documentao seja escrita
\emph{ao mesmo tempo que} o cdigo, e para isso nada melhor do que
usar o \emph{mesmo ficheiro} de texto para registar cdigo e documentao.
Como j se disse acima,
chama-se a esta estratgia \litp{\emph{programao literria}}, sendo o \Haskell\ uma
das linguagens em que tal estratgia  prtica comum.

 o que se passa com o texto (PDF) que neste momento est a ler: foi gerado a
partir de um ficheiro ( \file{cp0708t.lhs} ) usando o processador de texto
\Latex\ \cite{Kn86}; mas, se submeter o mesmo ficheiro ao seu interpretador
de \Haskell\ preferido, ver que o cdigo nele includo ser seleccionado,
carregado e executado.

Exemplo: vamos declarar de seguida os isomorfismos $in$ e $out$ associados
 definio de listas em \Haskell\ tal como foram definidos nas aulas 
desta disciplina.
Primeiro vamos dar um nome ao mdulo que estamos a definir
\begin{code}
module Cp0708t where
import Cp
import BTree
import LTree
import Ratio
import HugsList
import System.Time
\end{code}
aproveitando para indicar que ele depende de funes disponveis nos mdulos |Cp|,
|BTree|, etc.
Basta, ento escrevermos:
\begin{code}

inl = either (const []) (uncurry (:))

out []    = Left ()
out (a:x) = Right(a,x)
\end{code}

Se se inspeccionar o ficheiro \file{cp0708t.lhs} v-se
que todo o texto indentado acima est entre dois delimitadores,
\verb!\begin{code}! e \verb!\end{code}!.
 nisso que consiste o protocolo que o \Latex\ e o \Haskell\
estabelecem entre si: quando o \Latex\ encontra blocos 
\verb!\begin{code} ... \end{code}!, limita-se a mostr-los em formato \emph{pretty-printed}.
Por seu lado, o \Haskell\ est preparado para ignorar tudo o que  texto
\Latex\ \emph{e s prestar ateno} ao que est dentro desses blocos.

Por exemplo, se se submeter \file{cp0708t.lhs} ao \GHCi~\footnote{%
Note que tem de ter acessvel os ficheiros \file{Cp.hs}, etc que so importados.
Note ainda que deve invocar o \GHCi\ com as opes de comando
\texttt{-fglasgow-exts}
\texttt{-fallow-undecidable-instances}
\texttt{-fallow-overlapping-instances}.
}, ter-se- evidncia de que tudo correu bem,
\begin{verbatim}
   ___         ___ _
  / _ \ /\  /\/ __(_)
 / /_\// /_/ / /  | |      GHC Interactive, version 6.4.1, for Haskell 98.
/ /_\\/ __  / /___| |      http://www.haskell.org/ghc/
\____/\/ /_/\____/|_|      Type :? for help.

Loading package base-1.0 ... linking ... done.
Compiling Cp               ( ./Cp.hs, interpreted )
Compiling LTree            ( ./LTree.hs, interpreted )
Compiling HugsList         ( ./HugsList.hs, interpreted )
Compiling BTree            ( ./BTree.hs, interpreted )
Compiling Cp0708t          ( cp0708t.lhs, interpreted )
Ok, modules loaded: Cp0708t, BTree, HugsList, LTree, Cp.

\end{verbatim}
podendo-se de imediato questionar o interpretador.

Por exemplo, pode pedir-se para este calcular |out ["a","b","c"]|, devendo-se obter
\begin{verbatim}
*Cp0708t> out ["a","b","c"]
Right ("a",["b","c"])
\end{verbatim}
etc.

Continuando, vamos agora declarar o combinador $\cata\_$ que nos permite
construir \emph{catamorfismos} de listas:
\begin{code}
cata g   = g . rec (cata g) . out   
\end{code}
onde
\begin{code}
rec f    = id -|- id >< f
\end{code}
Voltando de novo ao \GHCi, podemos inquirir qual o tipo de |cata|,
\begin{verbatim}
*Cp0708t> :t cata
cata :: forall c a. (Either () (a, c) -> c) -> [a] -> c
\end{verbatim}
qual o resultado da aplicao do catamorfismo
\begin{eqnarray}
	\cata{\alt{\kons 0}{(1+) \p2}}
	\label{eq:070421a}
\end{eqnarray}
 lista acima,
\begin{verbatim}
*Cp0708t> cata (either (const 0) ((1+) . p2)) ["a","b","c"]
3
\end{verbatim}
etc.

\section{Como documentar clculos de programas}
Nas aulas desta disciplina, sempre que se pretende derivar um programa
em \Haskell, ou raciocinar sobre ele, no se usa a notao \Haskell\
directamente, mas sim uma notao mais compacta, de feio matemtica,
que permite abreviar texto a agilizar os clculos.
A expresso (\ref{eq:070421a}) acima, por exemplo,  a que  usada
no clculo de catamorfismos e no a expresso \Haskell\ que acima se
passou ao \GHCi. Se quisermos, podemos ver (\ref{eq:070421a}) como
o ``pretty-printing'' do referido texto em sintaxe \Haskell.

Assim, no vamos usar \litp{programao literria} para documentar os clculos,
mas sim o prprio \Latex, que tem uma natural afinidade com a matemtica.
De facto, a produo de texto em \Latex\  feita declarando funes (designadas
''macros'') cujo resultado  o texto a imprimir. Veja-se o exemplo
do operador unrio |cata|, que em \Latex\  a macro unria do mesmo
nome declarada no ficheiro auxiliar \file{cp0708t.sty}.

A inspeco deste ficheiro revelar ao leitor a vantagem de se reproduzir
na linguagem do processador de texto a estrutura da linguagem matemtica
que estamos a usar~\footnote{Em particular, a seco
\emph{Importing/modifying isolatin1} mostra como usar caracteres especiais
para esse efeito.}. Isso revela-se em particular no que diz respeito
 apresentao dos clculos, tal como o que se segue, extrado
``verbatim'' dos apontamentos tericos da disciplina~\cite{Ol05}:
\begin{calculation}
%
     swap  swap
%
\just={by definition $swap \deff \split{\p2}{\p1}$}
%
        \split{\p2}{\p1}  swap
%
\just={by $\times$-fusion (\ref{eq:702e})}
%
        \split{\p2  swap}{\p1  swap}
%
\just={definition of $swap$ twice}
%
        \split{\p2  \split{\p2}{\p1}}
	      {\p1  \split{\p2}{\p1}}
%
\just={by $\times$-cancellation (\ref{eq:700l})}
%
        \split{\p1}{\p2}
%
\just={by $\times$-reflexion (\ref{eq:703a})}
%
        id
%
\end{calculation}

\noindent
A inspeco de file{cp0708t.lhs} revelar uma estrutura
\begin{verbatim}
\begin{calculation}
%
     swap \comp swap
%
\just={ .................. }
%
	......
%
\just={ .................. }
%
        id
%
\end{calculation}
\end{verbatim}
cujo `layout' se aproxima bastante do que  produzido e onde desempenha papel
muito relevante a \emph{macro} binria \texcmd{just} (de \emph{``justificao''})
cujo primeiro argumento  o smbolo a justificar (igualdade de funes, neste caso)
e o segundo  o texto justificativo.

\section{O que se pretende}
O anexo \ref{sec:exercs} deste texto contm um enunciado de questes vrias
sobre a matria da disciplina, umas de natureza terica
e outras de natureza prtica, devendo resultar das primeiras clculos de
programas em estilo \emph{point-free} e das ltimas programas em \Haskell\
que devero poder ser testados num interpretador de \Haskell.
Os alunos devem responder a cada pergunta escrevendo o texto \lhaskell{lhs}
\litp{(``literate haskell'')}
que entenderem e como entenderem, ora escrevendo os seus clculos como acima se
explicou, ora embebendo cdigo \Haskell\ no seu texto.
Para imprimir o relatrio basta usar a sequncia de comandos
\begin{verbatim}
lhs2TeX -o cp0708t.tex cp0708t.lhs
pdflatex cp0708t
\end{verbatim}
(onde se usa \file{cp0708t.lhs} como exemplo) em que o papel essencial 
desempenhado pelo pre-processador \texttt{lhs2TeX}~\footnote{%
Ver \verb"www.informatik.uni-bonn.de/~loeh/lhs2tex".
}.

Sugere-se o aproveitamento da fonte \Latex\ deste documento (\file{cp0708t.lhs})
como ponto de partida. O ficheiro fonte do relatrio dever ser enviado
 equipa docente seguindo as instrues que sero publicadas na
pgina da disciplina.

Os alunos devero ainda aproveitar este texto para mostrar que sabem manipular
biliografias (em \Bibtex) e ndices/glossrios (usando \Makeindex).
Para se conhecer a vastssima documentao disponvel sobre \Latex,
\Bibtex, etc, etc, basta escrever \texttt{"latex"}, \texttt{"bibtex"},
\texttt{"makeindex"} etc num motor de busca e comear a navegar.
Ou ento visitar o \TUG\ (\emph{\TeX\ Users Group}) em
\texttt{www.tug.org/}.

\section{Desenho de diagramas}
H muitos \emph{pacotes \Latex} para desenhar diagramas e outros objectos
grficos \cite{GRM97}. Sugere-se, entre estes,
o \xypic{XY-pic}~\footnote{ Ver \texttt{http://www.tug.org/applications/Xy-pic/}.},
pela sua simplicidade e pelo facto de constar das principais distribuies
de \Latex. Como exemplo, d-se o diagrama,
\begin{eqnarray*}
\xymatrix{
	B
		\ar[d]_-{f}
&
	A
		\ar[l]_-{g}
		\ar[dl]^-{f g}
\\
	C
}
\end{eqnarray*}
extrado de  \cite{Ol05}.
Bastar inspeccionar o ficheiro \file{cp0708t.lhs} para
ver quo simples  desenhar diagramas como este.

\appendix

\newpage

\section{Exerccios a resolver}\label{sec:exercs}
Como orientao geral para a resoluo das questes que se seguem,
considerem que as respostas (sejam elas clculos, provas, cdigo \Haskell,
etc) sero tanto mais valorizadas quanto mais simples forem.
(No caso de cdigo \Haskell, procurem usar combinadores que reduzam o nmero
de linhas de cdigo.)

O nmero mnimo de alneas a resolver  8, das quais 4 so obrigatrias (marcadas
com o sinal ``\obrigatorio'').  As restantes 4 so  escolha de cada grupo.
As valorizaes de questes ou alneas obrigatrias, quando existam, so naturalmente
opcionais.

\traco

\begin{exercise} \obrigatorio
Demonstre a lei de fuso da exponenciao, cf.\ (\ref{eq:exp-fusion}) no Anexo
\ref{sec:FunCalcfragment}, pg.~ \pageref{eq:exp-fusion}, desenhando o diagrama
respectivo.
\end{exercise}

\begin{exercise} 
A funo exponencial $e^x : \R \rightarrow \R$ (onde ``$e$'' designa o nmero
de Euler) pode ser definida de vrias maneiras, entre elas aquela que toma
a forma de uma srie de Taylor:
\begin{eqnarray}
	e^x
& = &
	\sum_{n=0}^{\infty} \frac {x^n} {n!}
\end{eqnarray}
A seguinte funo, em \Haskell,
\begin{code}
euler :: Double -> Integer -> Double
euler x 0     = 1
euler x (n+1) = x .^ (n+1) / fac (n+1)  + (euler x n)
\end{code}
calcula uma aproximao a $e^x$, onde o parmetro adicional indica o nmero
de parcelas da srie de Taylor que esto a ser tomadas em considerao.
Por exemplo, enquanto que |euler 1 1 = 2.0|, j |euler 1 10| d o resultado
|2.7182818011463845|.

Recorde das aulas prticas que a exponenciao
\begin{code}
a .^ 0 = 1
a .^ (n+1) = (a .^ n) * a
\end{code}
 um catamorfismo de naturais, e que a funo factorial tambm se pode exprimir
atravs de um catamorfismo do mesmo tipo:
\begin{code}
fac = p2 . fcata where
           fcata 0 = (1,1)
           fcata (n+1) = let (a,b) = fcata n in (a+1,a*b)
\end{code}
Contudo, a funo |euler| no  muito eficiente.  Se recorrer s funes
disponveis na package |System.Time| para contagem de tempo de CPU verifica
que, enquanto o clculo de |euler 1 100|  quase instantneo, o de |euler 1 1000|
leva cerca de 9 segundos, o de |euler 1 2000| 33 segundos, etc
\begin{enumerate}
\item	\obrigatorio
	Apresente em detalhe todos os clculos que dever fazer para, recorrendo a
	leis como a de recursividade mltipla e/ou o seu corolrio \emph{banana-split},
	obter uma verso mais eficiente de |euler x n|.
\item
	Apresente resultados de testes de execuo das duas funes (variando o
	valor de |n|) por forma a documentar o ganho em eficincia.
	Para esse efeito, transforme-as em funes sobre o mnade de \IO\ e recorra,
	nesse contexto, a funes como |getClockTime|, |diffClockTimes| etc.\ da
	package |System.Time| que o ajudem nessa tarefa.
\end{enumerate}
\end{exercise}

\begin{exercise}
Foi estudada nas aulas prticas da disciplina a generalizao da lei distributiva
\begin{eqnarray}
	a * (b+c) &=& a * b + a * c
	\label{eq:080503a}
\end{eqnarray}
ao caso de $n$ parcelas,
\begin{eqnarray}
	a * \sum_{i=1}^{n} b_i &=& \sum_{i=1}^{n} (a * b_i)
\end{eqnarray}
que se pode traduzir em \Haskell\ da forma seguinte:
\begin{code}
distLaw a = [ (a *) . summation id , summation (a*) ]
\end{code}
onde
\begin{code}
summation f = cata (either (const 0)(uncurry(+) . (f >< id)))
\end{code}
%
Mostre que este resultado pode ser generalizado a catamorfismos de outras
estruturas como, por exemplo |LTree| ou |BTree|.
\\
\textbf{NB}: recorde das aulas a verso \emph{pointfree} de (\ref{eq:080503a}):
\begin{eqnarray}
	(a *) \comp \uncurry + & = & \uncurry + \comp ((a *) \times (a *))
\end{eqnarray}
\textbf{Valorizao}: poder optar por provar o resultado para o caso geral
envolvendo catamorfismos de um tipo genrico $\fun T\ A \ \iso \ \b(A, \fun T\ A)$.
\end{exercise}

\begin{exercise}
Defina catamorfismos de naturais que construam a srie $\{a_n\}$ e calculem
o somatrio do problema 3/4/19 do Round 4 proposto pela USAMTS 
(\verb!http://www.usamts.org/Problems/U_Problems.php!).
\end{exercise}

\begin{exercise}
Se inspeccionar, atravs do cdigo disponvel em \file{demos.hs} do material
pedaggico da disciplina, as estruturas intermdias (virtuais) dos hilomorfismos
\emph{hanoi} e \emph{qSort}, de tipo |BTree|, apercebe-se de propriedades
que essas estruturas tm, quaisquer que sejam os argumentos que sejam passados
quelas funes.

Propriedades desta natureza so habitualmente designadas \emph{invariantes}.
Codifique o teste dessas propriedades sobre a forma de hilomorfismos booleanos
(predicados) sobre |BTree|.
\end{exercise}

\begin{exercise}
Use a lei de fuso-cata instanciada para |LTree| na demonstrao da propriedade
seguinte,
\begin{eqnarray}
	countLTree  invLTree &=& countLTree
\end{eqnarray}
que nos diz que |countLTree|  invariante de |invLTree| (ie., o nmero de
folhas de uma |LTree| no se altera com o seu espelhamento).
\end{exercise}

\begin{exercise}
O matemtico francs douard Lucas (1842-1891), a quem se atribui a formulao
do problema das Torres de Hani, generalizou a srie de nmeros de Fibonacci
s chamadas \emph{sequncias de Lucas} (ver por exemplo
\verb!http://en.wikipedia.org/wiki/Lucas_sequence!).

Recorra s bibliotecas estudadas nesta disciplina para definir funes que
implementem o clculo de tais sequncias.
\end{exercise}

\begin{exercise}
Seja dado o tipo de dados, em \Haskell
\begin{code}
data PTree a b = Ind {
        name   :: a ,
        birth  :: b ,
        mother :: Maybe (PTree a b) ,
        father :: Maybe (PTree a b)
        } deriving (Show,Eq)
\end{code}
para registo de rvores genealgicas. Por exemplo, a rvore
\begin{eqnarray}\small
\xymatrix@@R=1em@@C=-2ex{
	\mbox{Jules, b.~1917}
&&
	\mbox{Margaret, b.~1923}
&
&
	\mbox{Luigi, b.~1920}
\\
&
	\mbox{Mary, b.~1956}
		\ar@@{-}[ul]
&
&
	\mbox{Joseph, b.~1955}
		\ar@@{-}[ul]
		\ar@@{-}[ur]
\\
&
&
	\mbox{Peter, b.~1991}
		\ar@@{-}[ul]
		\ar@@{-}[ur]
}
	\label{eq:070518a}
\end{eqnarray}
 representada pela |PTree|
\begin{code}
t = Ind {name = "Peter",
	 birth = 1991,
	 mother = Just (Ind {name = "Mary",
		 birth = 1956,
		 mother = Nothing,
		 father = Just (Ind {name = "Jules",
			 birth = 1917,
			 mother = Nothing,
			 father = Nothing})}),
		 father = Just (Ind {name = "Joseph",
		 birth = 1955,
		 mother = Just (Ind {name = "Margaret",
			 birth = 1923,
			 mother = Nothing,
			 father = Nothing}),
		 father = Just (Ind {name = "Luigi",
			 birth = 1920,
			 mother = Nothing,
			 father = Nothing})})}
\end{code}

\begin{enumerate}
\item	\obrigatorio
	Defina a lgebra
	|inPTree :: ((a, b), Maybe (PTree a b), Maybe (PTree a b)) -> PTree a b|,
	a sua conversa |outPTree| e ainda a triologia |anaPTree|, |cataPTree|, |hyloPTree|.
	Defina tambm |PTree| como instncia da classe |BiFunctor|.
	Use esses combinadores na formulao \emph{pointfree} das seguintes operaes:
	\begin{itemize}
	\item	contagem dos indivduos de uma |PTree|
	\item	transformao de uma dada |PTree| numa outra em que cada data de nascimento
		d lugar  idade actual do respectivo indivduo, mantendo-se o resto da
		informao.
	\end{itemize}
\textbf{Valorizao}: definir a operao que representa graficamente uma dada
	|PTree|; pode gerar essa visuali\-za\-o usando uma \emph{xymatrix} ---
	tal como em (\ref{eq:070518a}) --- ou recorrendo a um pacote grfico do
	seu agrado, eg.\ a biblioteca \SOEGraphics~\cite{Hu00}.

\item	
	Uma outra forma de representar rvores genealgicas consiste em
	associar identificadores nicos a cada indivduo (``BI''s) e us-los
	como referncias para os seus progenitores.
	Uma representao da rvore (\ref{eq:070518a}) neste tipo de formato
	poder ser, por exemplo, a estrutura
\begin{code}
h1 = Heap [
	1 |-> (("Peter",1991),Just 2,Just 3),
	2 |-> (("Mary",1956),Nothing,Just 6),
	6 |-> (("Jules",1917),Nothing,Nothing),
	3 |-> (("Joseph",1955),Just 5,Just 7),
	5 |-> (("Margaret",1923),Nothing,Nothing),
	7 |-> (("Luigi",1920),Nothing,Nothing)
  	  ] 1
\end{code}
	que habita o tipo
\begin{code}
data Heap a k = Heap [(k,(a,Maybe k, Maybe k))] k deriving Show
\end{code}
e onde se usa a notao
\begin{code}
x |-> y = (x,y)
\end{code}
para indicar a associao de informao a cada BI. Note que o construtor
|Heap|  binrio: o seu primeiro argumento  a lista associativa a que acima
se referiu. O segundo  o BI do indivduo que  a raiz da rvore
(|1|, no caso acima).

Para se ver que |h1| representa de facto |t| basta calcular
\begin{code}
teste = absf h1 == t
\end{code}
e verificar que |teste=True|, onde |absf|  a funo (parcial)
que converte |Heap| em |PTree|:
\begin{code}
absf :: (Eq k) => Heap (a, b) k -> PTree a b
absf (Heap h k) = let Just (a,x,y) = lookup k h
                  in  Ind (fst a)(snd a)
                           (fmap (absf . Heap h) x)
                           (fmap (absf . Heap h) y)
\end{code}
Contudo, nem todo o |Heap| representa uma rvore de tipo |PTree|
--- como se pode ver aplicando |absf| aos dois |Heap|s que se seguem,
\begin{code}
h2 = Heap [
	1 |-> (("Peter",1991),Just 2,Just 3),
	2 |-> (("Mary",1956),Nothing,Just 9),
	6 |-> (("Jules",1917),Nothing,Nothing),
	3 |-> (("Joseph",1955),Just 5,Just 7),
	5 |-> (("Margaret",1923),Nothing,Nothing),
	7 |-> (("Luigi",1920),Nothing,Nothing)
  	  ] 1
\end{code}
e
\begin{code}
h3 = Heap [
	1 |-> (("Peter",1991),Just 2,Just 3),
	2 |-> (("Mary",1956),Nothing,Just 6),
	6 |-> (("Jules",1917),Nothing,Nothing),
	3 |-> (("Joseph",1955),Just 5,Just 1),
	5 |-> (("Margaret",1923),Nothing,Nothing),
	7 |-> (("Luigi",1920),Nothing,Nothing)
  	  ] 1
\end{code}
	Identifique quais so os problemas em cada caso e re-escreva |absf| por
	forma a colmatar o seu comportamento deficiente.

\textbf{Valorizao}: investigue como definir a conversa de |absf|
--- isto , a funo |repf :: PTree a b -> Heap (a, b) Integer| que representa
rvores em \emph{heaps} --- sob a forma de um catamorfismo, isto , encontre o
	gene |g| tal que |repf = cataPTree g|.

\item
Declare |Heap| com instncia da classe |BiFunctor| por forma a que, em |bmap g f|,
|f| seja a funo que altera BIs. Com base nessa instncia, investigue 
propriedades que |f| deva satisfazer para que a rvore representada por um dado
|Heap| |h| seja a mesma que  representada por |bmap id g h|,
isto , tal que
\begin{center}
	|absf(bmap id g h) == absf h|
\end{center}
\end{enumerate}
\end{exercise}

\begin{exercise}
No seu livro \emph{The Haskell School of Expression} \cite{Hu00},
Paul Hudak d a seguinte sintaxe para a notao musical:

\begin{code}
data Music = Note Pitch Dur            -- a note (atomic object) 
           | Rest Dur                  -- a rest (atomic object)
           | Music :+: Music           -- sequential composition
           | Music :=: Music           -- parallel composition
           | Tempo  (Ratio Int) Music  -- scale the tempo
           | Trans  Int Music          -- transposition
           | Instr  IName Music        -- instrument label
    deriving (Show, Eq)
\end{code}
onde
\begin{code}
type Pitch      = (PitchClass, Octave)
data PitchClass = Cf | C | Cs | Df | D | Ds | Ef | E | Es | Ff | F | Fs
                | Gf | G | Gs | Af | A | As | Bf | B | Bs
     deriving (Eq,Show)
type Octave     = Int
type Dur   = Ratio Int                 -- in whole notes
type IName = String                    -- instrument name, eg. piano, flute etc
\end{code}

Apesar da sua simplicidade, esta sintaxe (abstracta)  suficiente para grafar
msica polifnica a vrias vozes e instrumentos, e para gerar ficheiros audveis
em formato MIDI arbitrariamente complexos~\footnote{Ver \texttt{http://www.haskell.org/haskore}.
 Por exemplo, o ficheiro \file{cp0708t.mid} que tem disponvel no material pedaggico
foi gerado a partir da converso para MIDI do trecho cuja sintaxe encontra
na varivel |s|. (Para no sobrecarregar o presente mdulo, essa varivel
est comentada neste documento; ter que ``descomentar'' o respectivo texto
para a ter acessvel em \LaTeX\ e em \Haskell.)}.
\begin{enumerate}
\item	O isomorfismo
\begin{code}
outMusic :: Music
            -> Either (Maybe Pitch, Dur) (Either (Una (Ratio Int) Int IName, Music) (Bin, (Music, Music)))
outMusic(Note p d)  = i1(Just p,d)
outMusic(Rest   d)  = i1(Nothing,d)
outMusic(m1 :+: m2)  = i2(i2(Seq,(m1,m2)))
outMusic(m1 :=: m2)  = i2(i2(Par,(m1,m2)))
outMusic(Tempo t m) =  i2(i1(Temp t,m))
outMusic(Trans t m) =  i2(i1(Tra  t,m))
outMusic(Instr t m) =  i2(i1(Ins  t,m))
\end{code}
onde
\begin{code}
data Bin = Seq | Par
data Una t t' i = Temp t | Tra t' | Ins i
\end{code}
 testemunha de que |Music|  um tipo polinomial do \esimo2 grau, com base
\begin{eqnarray}
	\b(A,B,C,X) &=& C + (B \times X + (A \times X^2))
	\label{eq:musicb}
\end{eqnarray}
isto , tem-se
\begin{code}
recMusic f = id -|- (id >< f -|- (id >< (f >< f)))
\end{code}
e
\begin{code}
cataMusic f = f . (recMusic (cataMusic f)) . outMusic
\end{code}
Considere o catamorfismo
\begin{code}
h = cataMusic (either (maybe 0 (const 1) . p1) (either id (uncurry (+)) . (p2 -|- p2)))
\end{code}
O que faz esta funo? Apresente exemplos informativos.
Escreva ainda como catamorfismo a funo que calcula a durao de um dado trecho musical.
\item
Complete a definio da biblioteca associada a |Music| (parametrizando-a) e
desenhe o diagrama dos respectivos hilomorfismos.
\end{enumerate}
\end{exercise}

\begin{exercise}
Na seco (5) da biblioteca \file{LTree.hs} declara-se o tipo |LTree a| como
instncia da classe |Monad|. Apresente clculos que provem que as propriedades
(\ref{eq:mon-mul}) e (\ref{eq:mon-unit}) so de facto vlidas, para este mnade.
\\
\textbf{Valorizao}: poder optar por provar que, em geral, qualquer tipo
cuja base  $\b(f,g) = f + \ff g$ (onde $\f$  um functor arbitrrio) forma
um mnade, para $\mu=\cata{\alt{id}{in \comp i_2}}$ e $\u=in \comp i_1$.
J agora: que outros mnades estudou que caem nesta classe?
\end{exercise}

\begin{exercise} \obrigatorio
Com base no exemplo que vem dado no ficheiro \file{mobile.hs}, escreva em
\Haskell\ um simulador de uma mquina de calcular que inclua um registo principal
(correspondente ao visor da mquina) e memria. Simule o pressionar de teclas
(eg.\ MC, M+, M-, etc) por comandos de |IO|, aos quais o simulador deve reagir
mostrando a actualizao do visor (registo principal).
\\
\textbf{Valorizao}: combine os mnades de estado  e de |IO| com o de tratamento
de erro por forma a lidar com operaes matematicamente erradas, por exemplo,
a sequncia "1", "/", "0", "=".
\end{exercise}
\newpage
\section{Algumas leis do clculo funcional}\label{sec:FunCalcfragment}

\subsection{Produto}
\begin{eqnarray}
\law{splitUniv}{
        Universal-$\times$
}{
	% diagrama
}{
k = \split f g & \equiv &
        \left\{
                \begin{array}{rcl}
                        \p1  k = f
                \\
                        \p2  k = g
                \end{array}
        \right.
}
\\
%
\law{700l}{
        Cancelamento-$\times$
}{
        diagrama
}{
        \p1  \split f g  = f
        & \hbox{,} &
        \p2  \split f g = g 
}
\\
\law{703a}{
	Reflexo-$\times$
}{
	diagrama
}{
	\split{\p1}{\p2} &=& id_{A\times B}
}
\\
\law{702e}{
        Fuso-$\times$
}{
        \xymatrix{
        A & A \times B \ar[l]_{\p1} \ar[r]^{\p2} & B
        \\ & C \ar[ul]^{g} \ar[u]^{\split g h} \ar[ur]_{h}
        \\ & D \ar[uul]^{g  f} \ar[u]^{f} \ar[uur]_{h  f}
}}{
        \split{g}{h}  f = \split{g  f}{h  f}
}
\\
\law{970511c}{
        Absoro-$\times$
}{
        \xymatrix{
           A & A \times B \ar[l]_{\p1} \ar[r]^{\p2} & B
        \\ D \ar[u]^{i} &
           D \times E \ar[l]_{\p1} \ar[r]^{\p2} \ar[u]^{i\times j} &
           E \ar[u]^{j}
        \\ & C \ar[ul]^{g} \ar[u]^{\split{g}{h}} \ar[ur]_{h}
}}{
	(i\times j)  \split{g}{h} = \split{i  g}{j  h}
}
\\
\law{extra1}{
        Def-$\times$
}{
diagrama
}{
        f \times g  = \split{f  \p1}{g  \p2}
}
\\
\law{extra5}{
        Natural-$\p1$
}{
}{
       \p1  (f \times g) &=& f  \p1
}
\\
\law{extra6}{
        Natural-$\p2$
}{
}{
       \p2  (f \times g) &=& g  \p2
}
\\
\law{705h}{
        Functor-$\times$
}{
}{
       (g  h) \times (i  j) &=&
       (g \times i)  (h \times j)
}
\\
\law{705j}{
        Functor-id-$\times$
}{
}{
        id_A \times id_B &=& id_{A\times B}
}
\\
\law{extra2}{
        Eq-$\times$
}{
diagrama
}{
        \split{f}{g}  = \split{h}{k} \; \equiv\; f=h \land g=k
}
\end{eqnarray}

\subsection{Coproduto}
\begin{eqnarray}
\law{eitherUniv}{
        Universal-$+$
}{
	% diagrama
}{
k = \alt f g & \equiv &
        \left\{
                \begin{array}{rcl}
                        k  i_1 = f
                \\
                        k  i_2 = g
                \end{array}
        \right.
}
\\
\law{701c}{
        Cancelamento-$+$
}{
        \xymatrix{
        A \ar[r]^{i_1} \ar[dr]_{g} &
        A+B \ar[d]^{\alt g h} &
        B \ar[l]_{i_2} \ar[dl]^{h}
        \\ & C
}
}{
        \alt g h  i_1 = g & , & \alt g h  i_2 = h
}
\\
\law{703b}{
        Reflexo-$+$ 
}{
        diagrama
}{
                \alt{i_1}{i_2} &=& id_{A + B}
}
\\
\law{+-fusion}{
        Fuso-$+$
}{
        \xymatrix{
        A \ar[r]^{i_1} \ar[dr]_{g} \ar[ddr]_{f\comp g} & A+B \ar[d]^{\alt g h} &
        B \ar[l]_{i_2} \ar[dl]^-{h} \ar[ddl]^{f\comp h}
        \\ & C \ar[d]^{f}
        \\ & D
}
}{
        f  \alt g h = \alt{f  g}{f  h}
}
\\
\law{+-absorption}{
        Absoro-$+$
}{
        \xymatrix{
           A \ar[r]^{i_1} \ar[d]_{i} & A+B \ar[d]_{i+j} & B \ar[l]_{i_2} \ar[d]^{j} &
        \\ D \ar[r]^{i_1} \ar[dr]_{g} & D+E \ar[d]^{\alt g h} &
           E \ar[l]^{i_2} \ar[dl]^{h}
        \\ & C
}
}{
        \alt g h  (i + j)  = \alt{g  i}{h  j}
}
\\
\law{extra3}{
        Def-$+$
}{
diagrama
}{
        f + g  = \alt{i_1  f}{ i_2  g}
}
\\
\law{706b}{
        Functor-$+$
}{
}{
        (g  h) + (i  j) = (g + i)  (h + j)
}
\\
\law{706d}{
        Functor-id-$+$
}{
}{   
        id_A + id_B = id_{A + B}
}
\\
\law{extra4}{
        Eq-$+$
}{
diagrama
}{
        \alt{f}{g}  = \alt{h}{k} \; \equiv\; f=h\, \land\, g=k
}
\\
\law{eq:701d}{
        Lei da troca
}{
}{
        \alt{\split f g}{\split h k} &=&
        \split{\alt f h}{\alt g k}
}
\end{eqnarray}

\subsection{Exponenciao}
\begin{eqnarray}
\law{transUniv}{
        Universal-exp
}{
}{   
	k = \trans f & \equiv & f = ap  (k \times id)
}
\\
\law{999f}{
        Cancelamento-exp
}{
        \xymatrix{
           B^A & B^A \times A \ar[r]^{ap} & B
        \\ C \ar[u]^{\trans f} &
           C \times A \ar[u]^{\trans f \times id} \ar[ru]_{f}
}
}{
        f = ap  (\trans f \times id)
}
\\
\law{exp-reflex}{
        Reflexo-exp
}{
        \xymatrix{
           B^A & B^A \times A \ar[r]^{ap} & B
        \\ B^A \ar[u]^{id_{B^A}} &
           B^A \times A \ar[u]^{id_{B^A} \times id_A} \ar[ru]_{ap}
}
}{
        \trans{ap} = id_{B^A}
}
\\
\law{exp-fusion}{
        Fuso-exp 
}{
	omitido
}{
        \trans{g\comp(f \times id)} = \trans{g}  f
}
\\
\law{999h}{
        Absoro-exp 
}{
	omitido
}{
	f^A  \trans g = \trans{f  g}
}{
}
\\
\law{080328a}{
        Def-exp 
}{
}{
	f^A = \trans{f  ap}
}{
}
\\
\law{expFun}{
        Functor-exp 
}{
}{
        (g  h)^A = g^A  h^A
}
\\
\law{expFunId}{
        Functor-id-exp
}{
}{
        id^A = id
}
\end{eqnarray}

\subsection{Mnadas}
\begin{eqnarray}
\law{mon-mul}{
        Multiplicao
}{
}{
	   =   \ff 
}
\\
\law{mon-unit}{
        Unidade
}{
}{
	  \u =   \ff \u = id
}
\end{eqnarray}

\section{Fonte de \file{cp0708t.mid}}
Omitida sob a forma de um comentrio \Latex.
\begin{comment}
\begin{code}
-- s = Instr "piano" ((((Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1)))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Trans 4 (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1))))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1)))))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1))))))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1)))))))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1))))))))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: Rest (0%1))))))))))))))))) 
-- 	:+: (Rest (1%16) 
-- 	:+: (((Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1)))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Trans 4 (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1))))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1)))))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1))))))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1)))))))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Note (C,4) (1%32) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Note (C,4) (1%32)) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (Rest (1%32) 
-- 	:+: Rest (0%1))))))))))))))))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: Rest (0%1))))))))))))))))) 
-- 	:+: (Rest (1%16) 
-- 	:+: Rest (0%1))))) 
-- 	:+: ((Rest (1%16) 
-- 	:+: ((Rest (7%8) 
-- 	:+: ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32)))) 
-- 	:+: (((Rest (3%4) 
-- 	:+: Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32)))) 
-- 	:+: (((Rest (5%8) 
-- 	:+: Trans 4 (Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))))) 
-- 	:+: (((Rest (1%2) 
-- 	:+: Trans 4 (Trans 4 (Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32)))))) 
-- 	:+: (((Rest (3%8) 
-- 	:+: Trans 4 (Trans 4 (Trans 4 (Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))))))) 
-- 	:+: (((Rest (1%4) 
-- 	:+: Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32)))))))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))))))))) 
-- 	:+: ((Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%8)))) 
-- 	:+: Rest (1%8))) 
-- 	:+: Rest (1%8))) 
-- 	:+: Rest (1%8))) 
-- 	:+: Rest (1%8))) 
-- 	:+: Rest (1%8))) 
-- 	:+: Rest (1%8))) 
-- 	:+: Rest (1%8)))) 
-- 	:+: ((((Rest (7%8) 
-- 	:+: ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32)))) 
-- 	:+: (((Rest (3%4) 
-- 	:+: Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32)))) 
-- 	:+: (((Rest (5%8) 
-- 	:+: Trans 4 (Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))))) 
-- 	:+: (((Rest (1%2) 
-- 	:+: Trans 4 (Trans 4 (Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32)))))) 
-- 	:+: (((Rest (3%8) 
-- 	:+: Trans 4 (Trans 4 (Trans 4 (Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))))))) 
-- 	:+: (((Rest (1%4) 
-- 	:+: Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32)))))))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))))))))) 
-- 	:+: ((Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 (Trans 4 ((Rest (7%32) 
-- 	:+: Note (C,4) (1%32)) 
-- 	:+: (((Rest (3%16) 
-- 	:+: Trans 5 (Note (C,4) (1%32))) 
-- 	:+: (((Rest (5%32) 
-- 	:+: Trans 5 (Trans 5 (Note (C,4) (1%32)))) 
-- 	:+: (((Rest (1%8) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))) 
-- 	:+: (((Rest (3%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))) 
-- 	:+: (((Rest (1%16) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))) 
-- 	:+: (((Rest (1%32) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32)))))))) 
-- 	:+: (((Rest (0%1) 
-- 	:+: Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Trans 5 (Note (C,4) (1%32))))))))) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))) 
-- 	:+: Rest (1%32))))))))) 
-- 	:+: (Rest (1%8) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%8)))) 
-- 	:+: Rest (1%8))) 
-- 	:+: Rest (1%8))) 
-- 	:+: Rest (1%8))) 
-- 	:+: Rest (1%8))) 
-- 	:+: Rest (1%8))) 
-- 	:+: Rest (1%8))) 
-- 	:+: Rest (1%8))) 
-- 	:+: (Rest (17%16) 
-- 	:+: (Rest (0%1) 
-- 	:+: Rest (1%16)))) 
-- 	:+: Rest (1%16))))
\end{code}
\end{comment}
%----------------- Bibliografia (exige bibtex) --------------------------------%
\newpage
\bibliographystyle{plain}
\bibliography{cp0708t}
%----------------- ndice remissivo (exige makeindex) -------------------------%
\printindex
\end{document}

