
-- (c) MP-I and CP (1998/99-2007/08)

import St
import Cp

-- === Stack: ======================================================================

-- (a) functional layer ----------------------------------------

pushf =  uncurry(:) 

popf  = tail

topf  = head

emptyf = (0==) . length

start = []

-- (b) reactive layer (St monadic) ------------------------------

push :: a -> St [a] ()
push = St . (curry (split (!) id . pushf))

pop :: () -> St [a] a
pop  = St . (curry (split topf popf . p2))  -- two (partial) functions merged in one action

empty :: () -> St [a] Bool                  -- predicate promoted to action
empty() = query emptyf

-- (c) behavioural semantics (example of law) -------------------

fact = [ pop .! push , return]  -- pop .! push = return

-- (d) monadic run example --------------------------------------

-- naive simulation of typical (non optimized) state monadic code
-- compiled for evaluating expression
--
--              x * ( y + 2)
--
-- where x and y are global variables (holding 10 and 11 in this run):

run = exec prog start
      where prog = do { -- loading
                        push(x);
                        push(2);
                        push(y);
                        -- evaluating y + 2
                        r1 <- pop();
                        r2 <- pop();
                        push(r1+r2);
                        -- evaluating x * (y + 2)
                        r1 <- pop();
                        r2 <- pop();
                        push(r1*r2);
                        -- get returns current state
                        get
                      } where -- sample data
                              x = 10
                              y = 11

-- === Word count automata: ========================================================

-- (a) original program in C -----------
 
-- 1 #define YES 1
-- 2 #define NO 0
-- 3 main()
-- 4 {
-- 5 int c, nw, inword ;
-- 6 inword = NO ;
-- 8 nw = 0;
-- 10 c = getchar();
-- 11 while ( c != EOF ) {
-- 15     if ( c == ' ' || c == '\n' || c == '\t')
-- 16           inword = NO;
-- 17     else if (inword == NO) {
-- 18           inword = YES ;
-- 19           nw = nw + 1;
-- 20     }
-- 21     c = getchar();
-- 22 }
-- 24 printf("%d",nw);
-- 26 }

-- (b) extracting the automaton behind this piece of code

-- state is pair (inword,nw)
-- delta is the transition function:

deltaf :: (Num b) => (Char, (Bool, b)) -> ((), (Bool, b))
-- case inword is false
deltaf(c,(False,nw)) = ((),(if sep c then  False      else  True,nw))
-- case inword is true 
deltaf(c,(True,nw)) = ((), if sep c then (False,nw+1) else (True,nw))

-- aux function:

sep c = ( c == ' ' || c == '\n' || c == '\t')

-- delta goes monadic

delta = St . (curry deltaf) 

-- test run illustrates use of sequence_ :: Monad m => [m a] -> m ()

run1 = let input = "A Maria comeu a laranja "
           prog = sequence_ (map delta input) >> get
       in  (p2.p2.(exec prog)) (False,0)


-- (c) functional (stateless) equivalent in Haskell

nw []    = 0
nw (c:l) = if not (sep c) && lookahead_sep l
           then nw l + 1 else nw l
           where lookahead_sep []    = True
                 lookahead_sep (c:l) = sep c

-- test run for nw

run2 = let input = "A Maria comeu a laranja "
       in nw input

-- === Becoming addicted to state ... ===================================================
-- cf. Haskell doc

tick :: St Int ()
tick = do n <- get
          put (n+1)
          return ()

incr :: Int -> Int
incr = p2 . (exec tick)



